/**
 * Copyright (c) 2015 - present LibDriver All rights reserved
 * 
 * The MIT License (MIT)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE. 
 *
 * @file      driver_w25qxx_interface_template.c
 * @brief     driver w25qxx interface template source file
 * @version   1.0.0
 * @author    Shifeng Li
 * @date      2021-07-15
 *
 * <h3>history</h3>
 * <table>
 * <tr><th>Date        <th>Version  <th>Author      <th>Description
 * <tr><td>2021/07/15  <td>1.0      <td>Shifeng Li  <td>first upload
 * </table>
 */

#include <quadspi.h>

#include "driver_w25qxx_interface.h"
/**
 * @brief  interface spi qspi bus init
 * @return status code
 *         - 0 success
 *         - 1 spi qspi init failed
 * @note   none
 */
uint8_t w25qxx_interface_spi_qspi_init(void)
{
    return 0;
}

/**
 * @brief  interface spi qspi bus deinit
 * @return status code
 *         - 0 success
 *         - 1 spi qspi deinit failed
 * @note   none
 */
uint8_t w25qxx_interface_spi_qspi_deinit(void)
{
    return 0;
}

/**
 * @brief      interface spi qspi bus write read
 * @param[in]  instruction sent instruction
 * @param[in]  instruction_line instruction phy lines
 * @param[in]  address register address
 * @param[in]  address_line address phy lines
 * @param[in]  address_len address length
 * @param[in]  alternate register address
 * @param[in]  alternate_line alternate phy lines
 * @param[in]  alternate_len alternate length
 * @param[in]  dummy dummy cycle
 * @param[in]  *in_buf pointer to a input buffer
 * @param[in]  in_len input length
 * @param[out] *out_buf pointer to a output buffer
 * @param[in]  out_len output length
 * @param[in]  data_line data phy lines
 * @return     status code
 *             - 0 success
 *             - 1 write read failed
 * @note       none
 */
uint8_t w25qxx_interface_spi_qspi_write_read(uint8_t instruction, uint8_t instruction_line,
                                             uint32_t address, uint8_t address_line, uint8_t address_len,
                                             uint32_t alternate, uint8_t alternate_line, uint8_t alternate_len,
                                             uint8_t dummy, uint8_t *in_buf, uint32_t in_len,
                                             uint8_t *out_buf, uint32_t out_len, uint8_t data_line)
{
    QSPI_CommandTypeDef cmd_handler;
    
    /* check the len */
    if ((in_len != 0) && (out_len != 0))
    {
        return 2;
    }
    
    /* set the instruction */
    cmd_handler.Instruction = instruction;
    if (instruction_line == 0)
    {
        cmd_handler.InstructionMode = QSPI_INSTRUCTION_NONE;
    }
    else if (instruction_line == 1)
    {
        cmd_handler.InstructionMode = QSPI_INSTRUCTION_1_LINE;
    }
    else if (instruction_line == 2)
    {
        cmd_handler.InstructionMode = QSPI_INSTRUCTION_2_LINES;
    }
    else if (instruction_line == 4)
    {
        cmd_handler.InstructionMode = QSPI_INSTRUCTION_4_LINES;
    }
    else
    {
        return 2;
    }
    
    /* set the address_line */
    if (address_line == 0)
    {
        cmd_handler.AddressMode = QSPI_ADDRESS_NONE;
    }
    else if (address_line == 1)
    {
        cmd_handler.AddressMode = QSPI_ADDRESS_1_LINE;
    }
    else if (address_line == 2)
    {
        cmd_handler.AddressMode = QSPI_ADDRESS_2_LINES;
    }
    else if (address_line == 4)
    {
        cmd_handler.AddressMode = QSPI_ADDRESS_4_LINES;
    }
    else
    {
        return 2;
    }
    
    /* set the address_len */
    if (address_len == 0)
    {
        cmd_handler.Address = address & 0xFF;
        cmd_handler.AddressSize = QSPI_ADDRESS_8_BITS;
    }
    else if (address_len == 1)
    {
        cmd_handler.Address = address & 0xFF;
        cmd_handler.AddressSize = QSPI_ADDRESS_8_BITS;
    }
    else if (address_len == 2)
    {
        cmd_handler.Address = address & 0xFFFF;
        cmd_handler.AddressSize = QSPI_ADDRESS_16_BITS;
    }
    else if (address_len == 3)
    {
        cmd_handler.Address = address & 0xFFFFFF;
        cmd_handler.AddressSize = QSPI_ADDRESS_24_BITS;
    }
    else if (address_len == 4)
    {
        cmd_handler.Address = address & 0xFFFFFFFF;
        cmd_handler.AddressSize = QSPI_ADDRESS_32_BITS;
    }
    else
    {
        return 2;
    }
    
    /* set the alternate */
    cmd_handler.AlternateBytes = alternate;
    if (alternate_line == 0)
    {
        cmd_handler.AlternateByteMode = QSPI_ALTERNATE_BYTES_NONE;
    }
    else if (alternate_line == 1)
    {
        cmd_handler.AlternateByteMode = QSPI_ALTERNATE_BYTES_1_LINE;
    }
    else if (alternate_line == 2)
    {
        cmd_handler.AlternateByteMode = QSPI_ALTERNATE_BYTES_2_LINES;
    }
    else if (alternate_line == 4)
    {
        cmd_handler.AlternateByteMode = QSPI_ALTERNATE_BYTES_4_LINES;
    }
    else
    {
        return 2;
    }
    
    /* set the alternate_len */
    if (alternate_len == 0)
    {
        cmd_handler.AlternateBytesSize = QSPI_ALTERNATE_BYTES_8_BITS;
    }
    else if (alternate_len == 1)
    {
        cmd_handler.AlternateBytesSize = QSPI_ALTERNATE_BYTES_8_BITS;
    }
    else if (alternate_len == 2)
    {
        cmd_handler.AlternateBytesSize = QSPI_ALTERNATE_BYTES_16_BITS;
    }
    else if (alternate_len == 3)
    {
        cmd_handler.AlternateBytesSize = QSPI_ALTERNATE_BYTES_24_BITS;
    }
    else if (alternate_len == 4)
    {
        cmd_handler.AlternateBytesSize = QSPI_ALTERNATE_BYTES_32_BITS;
    }
    else
    {
        return 2;
    }
    
    /* set the dummy */
    cmd_handler.DummyCycles = dummy;
    
    /* set the data_line */
    if (data_line == 0)
    {
        cmd_handler.DataMode = QSPI_DATA_NONE;
    }
    else if (data_line == 1)
    {
        cmd_handler.DataMode = QSPI_DATA_1_LINE;
    }
    else if (data_line == 2)
    {
        cmd_handler.DataMode = QSPI_DATA_2_LINES;
    }
    else if (data_line == 4)
    {
        cmd_handler.DataMode = QSPI_DATA_4_LINES;
    }
    else
    {
        return 2;
    }
    
    /* set the param */
    cmd_handler.SIOOMode = QSPI_SIOO_INST_EVERY_CMD;
    cmd_handler.DdrMode = QSPI_DDR_MODE_DISABLE;
    cmd_handler.DdrHoldHalfCycle = QSPI_DDR_HHC_ANALOG_DELAY;
    
    /* set the qspi */
    if (HAL_QSPI_Command(&hqspi, &cmd_handler, 1000) != HAL_OK)
    {
        return 1;
    }
    
    /* if input */
    if (in_len != 0)
    {
        hqspi.Instance->DLR = in_len - 1;
        if (HAL_QSPI_Transmit(&hqspi, in_buf, 1000) != HAL_OK)
        {
            return 1;
        }
    }
    
    /* if output */
    if (out_len != 0)
    {
        hqspi.Instance->DLR = out_len - 1;
        if (HAL_QSPI_Receive(&hqspi, out_buf, 1000) != HAL_OK)
        {
            return 1;
        }
    }
    
    return 0;
}

/**
 * @brief     interface delay ms
 * @param[in] ms time
 * @note      none
 */
void w25qxx_interface_delay_ms(uint32_t ms)
{
    HAL_Delay(ms);
}

/**
 * @brief     interface delay us
 * @param[in] us time
 * @note      none
 */
void w25qxx_interface_delay_us(uint32_t us)
{

}

/**
 * @brief     interface print format data
 * @param[in] fmt format data
 * @note      none
 */
void w25qxx_interface_debug_print(const char *const fmt, ...)
{
    
}
